/*Class definition: This class essentially processes the "move" made by the opponent as
 * well as Harry and responds according. This has the combat mode defined in it. It also has
 * methods for general display of menus, getting input for menu choice and checking if the character can
 * in fact make the move on the sector.
 */
import java.util.Random;
import java.util.Scanner;

public class CommandProcessor {
	TriwizardMaze maze = new TriwizardMaze();
	GameStatus currentGame = new GameStatus();
	//The very beginning of the game. Displays the maze, the gameStatus, processes Harry's as
	//well as the opponent's movement
	public void start() {
		do {
			maze.displayMaze();
			displayGameStatus();
			processHarryMove();
			processOpponentMove();
		} while (!(currentGame.gameLost || currentGame.gameWon || currentGame.gameQuit));
	}
/*Prints Harry's status as well as the opponents's status. It also prints the latest
 * developments in the game (Who got charmed, was there an attack!! I better run!)
 */

	public void displayGameStatus() {
		System.out.println("STATUS BAR: ");
		System.out.println(maze.player);
		for (int index = 0; index < maze.opponents.size(); index++) {
			System.out.print(maze.opponents.get(index) + "\t");
		}
		if (currentGame.latestStatus != null) {
			System.out.println("\n" + currentGame.latestStatus);
		}
		System.out.println();
		currentGame.latestStatus = null;

	}
/*Checks if the opponent can attack (if it is in an adjacent cell). Remember
 * the opponent can't attack if it is charmed/blinded.
 */
	private void processOpponentMove() {


		for (int i = 0; i < maze.opponents.size(); i++) {
			Adversary opponent = maze.opponents.get(i);
			opponent.wakeUp();

			int dX = opponent.xPos - maze.player.xPos;
			int dY = opponent.yPos - maze.player.yPos;

			if (dX >= -1 && dX <= 1 && dY >= -1 && dY <= 1) {
				if (currentGame.cheatModeOn == false
						&& opponent.isBlind() == false
						&& opponent.isCharmed() == false) {
					maze.player.processDamage(opponent.attack());
					currentGame.latestStatus += "YOU GOT ATTACKED BY "
							+ opponent.name + "!!\n";
				}
			} else {
				if (opponent.isBlind() == false) {
					processRandomMovement(opponent);
				}
			}
			//Check if Harry has used up all his health. If yes, then Sigh... You have lost
			if (maze.player.health <= 0) {
				currentGame.gameLost = true;
				currentGame.latestStatus += "YOU ARE DEAD, YOU LOSE !!\n";
				return;
			}
		}

	}
	//Generates the random movement for a character. The concept behind attempts is if
	//any character is cornered (a dementor at the edge of the maze with Artefacts all around)
	//then it will attempt to move a fixed number of times (8x2). If it cant move then it just gives
	//up stays where it is
	private void processRandomMovement(GameCharacter opponent) {
		int attempts = 0;
		Random rand = new Random();
		while (attempts < 16) {
			int move = 1 + rand.nextInt(8);
			//First checks if opponent is allowed to move to the sector
			if (canMove(opponent, move)) {
				opponent.xPos += maze.xMove[move];
				opponent.yPos += maze.yMove[move];
				break;
			}
			attempts++;
		}
	}
	//Checks if the Character can make a move on the destination (If the sector is not occupied
	// or if the sector is not outside the bounds of the array)
	private boolean canMove(GameCharacter obj1, int move) {
		int destX = obj1.xPos + maze.xMove[move];
		int destY = obj1.yPos + maze.yMove[move];

		if (!maze.withinGrid(destX, destY))
			return false;

		GameBaseObject obj2 = maze.grid[destX][destY];
		if (obj2 == null) {
			return true;
		} else if (obj1 instanceof HarryPotter && obj2 instanceof Artefact) {
			return true;
		}
		return false;
	}
	//Process the move made by Harry and act accordingly. For example if the move is to attack
	//display attack menu. If choice is to cast a spell then display spell menu.
	private void processHarryMove() {
		int choice, subchoice;
		Move move = null;
		boolean moveComplete = false;
		displayMenu('M');
		choice = getInput();
		move = processMove(maze.player, choice, 9);
		do {
			switch (move) {
			case ATTACK:
				displayMenu('A');
				do {
					subchoice = getInput();
				} while (subchoice > 2);
				if (subchoice == 1) {
					move = Move.COMBAT;
				} else if (subchoice == 2) {
					move = Move.SPELL;
				} else {
					move = processMove(maze.player, subchoice, 2);
				}
				break;
			case COMBAT:
				invokePlayerAttack(choice, Move.COMBAT);
				moveComplete = true;
				break;
			case SPELL:
				displayMenu('S');
				do {
					subchoice = getInput();
				} while (subchoice > 2);
				if (subchoice == 1) {
					move = Move.BLIND;
				} else if (subchoice == 2) {
					move = Move.CHARM;
				} else {
					move = processMove(maze.player, subchoice, 2);
				}
				break;
			case BLIND:
				invokePlayerAttack(choice, Move.BLIND);
				moveComplete = true;
				break;
			case CHARM:
				invokePlayerAttack(choice, Move.CHARM);
				moveComplete = true;
				break;
			case PICKUP:
				invokePickUp(maze.player, choice);
				maze.player.xPos += maze.xMove[choice];
				maze.player.yPos += maze.yMove[choice];
				moveComplete = true;
				break;
			case CHEAT:
				displayMenu('C');
				currentGame.cheatModeOn = (currentGame.cheatModeOn) ? false
						: true;
				moveComplete = true;
				break;
			case MOVEMENT:
				maze.player.xPos += maze.xMove[choice];
				maze.player.yPos += maze.yMove[choice];
				moveComplete = true;
				break;
			case QUIT:
				currentGame.gameQuit = true;
				moveComplete = true;
				break;
			default:
				displayMenu('M');
				do {
					choice = getInput();
					move = processMove(maze.player, choice, 9);
				} while (move == Move.INVALID);

				break;

			}
		} while (moveComplete == false);
	}

	private void invokePickUp(HarryPotter player, int choice) {
		int destX = maze.player.xPos + maze.xMove[choice];
		int destY = maze.player.yPos + maze.yMove[choice];

		for (int index = 0; index < maze.artefacts.size(); index++) {
			Artefact artefact = maze.artefacts.get(index);

			if ((artefact.xPos == destX) && (artefact.yPos == destY)) {
				artefact.pickObject(player);
				if (artefact instanceof TriwizardCup) {
					currentGame.latestStatus += "You WON !!\n";
					currentGame.gameWon = true;
				}
				maze.artefacts.remove(artefact);
				currentGame.latestStatus += "You picked up: " + artefact + "\n";
				break;
			}
		}
	}
	//Process the menu choice as well as the move
	private Move processMove(GameCharacter player, int choice, int limit) {
		//Choice 0 would invoke Cheat Menu
		if (choice == 0) {
			return Move.CHEAT;
		}
		//Choice < 0 would Quit the game
		if (choice < 0) {
			return Move.QUIT;
		}
		//Any other choice > limit would be invalid
		if (choice > limit) {
			return Move.INVALID;
		}
		//Get the x and y coordinates of the new position based on menu choice
		int destX = player.xPos + maze.xMove[choice];
		int destY = player.yPos + maze.yMove[choice];

		if (!maze.withinGrid(destX, destY))
			return Move.INVALID;
		//Get the object at the coordinates
		//If object is null, then move. If object is an Adversary, then attack. If object is an
		//Artefact then pick up
		GameBaseObject obj = maze.grid[destX][destY];
		if (obj == null) {
			return Move.MOVEMENT;
		} else if (obj instanceof Adversary) {
			return Move.ATTACK;
		} else if (obj instanceof Artefact) {
			return Move.PICKUP;
		}
		return Move.INVALID;

	}
	//displays the appropriate menu
	private void displayMenu(char phase) {

		switch (phase) {
		case 'A':
			System.out.println("COMBAT (1) or CAST SPELL (2)");
			break;
		case 'S':
			System.out.println("BLIND (1) or CHARM (2)");
			break;
		case 'C':
			System.out.println("CHEAT ENABLED: Invincibility On");
			break;
		case 'M':
			System.out.println("MOVEMENT MENU: OPTIONS");
			System.out.println("7 8 9\n4   6\n1 2 3");
			System.out.println("To quit the game, enter a negative value.");
			break;
		default:
			System.out.println("Worng choice!");
			break;
		}
	}
	//Get the input for menu choice
	private int getInput() {
		System.out.print("Choice:? ");
		Scanner input = new Scanner(System.in);
		int getInput = input.nextInt();
		return getInput;
	}
	//This controls the attack mode. This acts depending upon the attack type and depending upon the
	//opponent. For example, you can neither charm nor blind a dementor.
	private void invokePlayerAttack(int moveDirection, Move attackType) {

		int destX = maze.player.xPos + maze.xMove[moveDirection];
		int destY = maze.player.yPos + maze.yMove[moveDirection];

		// For every opponent
		for (int index = 0; index < maze.opponents.size(); index++) {

			Adversary opponent = maze.opponents.get(index);

			if ((opponent.xPos == destX) && (opponent.yPos == destY)) {
				if (attackType == Move.CHARM || attackType == Move.BLIND) {
					if (maze.player.castSpell()) {
						currentGame.latestStatus += "You cast a spell on "
								+ opponent.name + "!!\n";
						boolean spellSucess = opponent.getSpellCast(attackType);
						if (spellSucess && attackType == Move.CHARM) {
							currentGame.latestStatus += "You charmed  "
									+ opponent.name + "!!\n";
						} else if (spellSucess && attackType == Move.BLIND) {
							currentGame.latestStatus += "You blinded  "
									+ opponent.name + "!!\n";
						} else {
							currentGame.latestStatus += "Your spell did not work on "
									+ opponent.name + "!!\n";
						}
					} else {
						currentGame.latestStatus += "You don't have enough Spell points!!\n";
					}
				} else if (attackType == Move.COMBAT) {
					int combatPoints = maze.player.attack();
					opponent.processDamage(combatPoints);
					currentGame.latestStatus += "You attacked " + opponent.name
							+ "!!\n";
				}
				//Check if opponent has <= 0 health. If that is in fact the case, hurray! You have
				//killed the opponent
				if (opponent.health <= 0) {
					currentGame.latestStatus += "You KILLED " + opponent.name
							+ "!!\n";
					maze.opponents.remove(opponent);
				}
				break;
			}

		}

	}

}