/*Class definition: This class initializes the TriWizard maze. It also has methods
 * for refreshing, clearing as well displaying the maze.
 */
import java.util.ArrayList;
import java.util.Random;

public class TriwizardMaze {

	// Initializing the maze. You can play with these parameters to make the game more challenging
	//Eg. You can increase the number of bad guys!
	public static final int SIZE = 10;
	private static final int HARRY_X_POS = 0;
	private static final int HARRY_Y_POS = 0;
	private static final int MAX_HEALTH_OBJECTS = 3;
	private static final int MAX_SCROLL_OBJECTS = 3;
	private static final int MAX_DEFENCE_OBJECTS = 3;
	private static final int MAX_ATTACK_OBJECTS = 3;
	private static final int MAX_OPPONENTS_BOGGARTS = 3;
	private static final int MAX_OPPONENTS_BES = 3;
	private static final int MAX_OPPONENTS_DEMENTORS = 2;

	final int xMove[] = { 0, 1, 1, 1, 0, 0, 0, -1, -1, -1 };
	final int yMove[] = { 0, -1, 0, 1, -1, 0, 1, -1, 0, 1 };

	GameBaseObject[][] grid;

	HarryPotter player;
	//ArrayList for maintaining list of artefacts and bad guys
	ArrayList<Artefact> artefacts;
	ArrayList<Adversary> opponents;

	public TriwizardMaze() {
		initializeMaze();
		//Creating a new player (HarryPotter object) and placing it at the initial position
		//in the grid (0,0)
		player = new HarryPotter("Harry", 'H', HARRY_X_POS, HARRY_Y_POS);
		grid[HARRY_X_POS][HARRY_Y_POS] = player;
		//Creating the TriwizardCup object and placing it at the end of the maze
		TriwizardCup ticketOutOfHere = new TriwizardCup(SIZE - 1, SIZE - 1);
		grid[SIZE - 1][SIZE - 1] = ticketOutOfHere;
		//Adding the TriwizardCup to the list of artefacts
		artefacts.add(ticketOutOfHere);

		// Create an object for generation of random numbers
		Random rand = new Random();
		int randomX, randomY;

		// place health artefacts at random points and adding them to the artefacts list
		for (int index = 0; index < MAX_HEALTH_OBJECTS; index++) {
			do {
				randomX = rand.nextInt(SIZE);
				randomY = rand.nextInt(SIZE);
			} while (grid[randomX][randomY] != null);
			Health artefact = new Health(randomX, randomY);
			grid[randomX][randomY] = artefact;
			artefacts.add(artefact);
		}

		// place scroll artefacts at random points and adding them to the artefacts list
		for (int index = 0; index < MAX_SCROLL_OBJECTS; index++) {
			do {
				randomX = rand.nextInt(SIZE);
				randomY = rand.nextInt(SIZE);
			} while (grid[randomX][randomY] != null);
			Scroll artefact = new Scroll(randomX, randomY);
			grid[randomX][randomY] = artefact;
			artefacts.add(artefact);
		}

		// place sword artefacts at random points and adding them to the artefacts list
		for (int index = 0; index < MAX_ATTACK_OBJECTS; index++) {
			do {
				randomX = rand.nextInt(SIZE);
				randomY = rand.nextInt(SIZE);
			} while (grid[randomX][randomY] != null);
			Sword artefact = new Sword(randomX, randomY);
			grid[randomX][randomY] = artefact;
			artefacts.add(artefact);
		}

		// place shield artefacts at random points and adding them to the artefacts list
		for (int index = 0; index < MAX_DEFENCE_OBJECTS; index++) {
			do {
				randomX = rand.nextInt(SIZE);
				randomY = rand.nextInt(SIZE);
			} while (grid[randomX][randomY] != null);
			Shield artefact = new Shield(randomX, randomY);
			grid[randomX][randomY] = artefact;
			artefacts.add(artefact);
		}

		// Place Boggarts at random points and adding them to the opponents list
		for (int index = 0; index < MAX_OPPONENTS_BOGGARTS; index++) {
			do {
				randomX = rand.nextInt(SIZE);
				randomY = rand.nextInt(SIZE);
			} while (grid[randomX][randomY] != null);
			Adversary opponent = new Boggarts("Boggart" + (index + 1), 'B',
					randomX, randomY);
			grid[randomX][randomY] = opponent;
			opponents.add(opponent);
		}

		// Place Blast-ended Skrewts at random points and adding them to the opponents list
		for (int index = 0; index < MAX_OPPONENTS_BES; index++) {
			do {
				randomX = rand.nextInt(SIZE);
				randomY = rand.nextInt(SIZE);
			} while (grid[randomX][randomY] != null);
			Adversary opponent = new BlastEndedSkrewts(
					"BlastEndedSkrewt" + (index + 1), 'S', randomX, randomY);
			grid[randomX][randomY] = opponent;
			opponents.add(opponent);
		}

		// Place Dementors at random points and adding them to the opponents list
		for (int index = 0; index < MAX_OPPONENTS_DEMENTORS; index++) {
			do {
				randomX = rand.nextInt(SIZE);
				randomY = rand.nextInt(SIZE);
			} while (grid[randomX][randomY] != null);
			Adversary opponent = new Dementors("Dementor" + (index + 1), 'D',
					randomX, randomY);
			grid[randomX][randomY] = opponent;
			opponents.add(opponent);
		}

	}
	//Initialzing the maze
	private void initializeMaze() {
		grid = new GameBaseObject[SIZE][SIZE];
		opponents = new ArrayList<Adversary>();
		artefacts = new ArrayList<Artefact>();
		clearGrid();
	}
	//Clearing the grid (Setting null to each sector)
	private void clearGrid() {
		for (int i = 0; i < SIZE; i++) {
			for (int j = 0; j < SIZE; j++) {
				grid[i][j] = null;
			}
		}
	}
	//Updating the maze (2-D array of references to GameBaseObject grid)
	private void refreshGrid() {
		clearGrid();
		//Checking if Harry is still alive (health > 0). If yes, then add to grid
		if (player.health > 0)
			grid[player.xPos][player.yPos] = player;
		//Checking if the opponents are still alive. If yes, then adding them to grid
		for (int index = 0; index < opponents.size(); index++) {
			GameCharacter opponent = opponents.get(index);
			if (opponent.health > 0) {
				grid[opponent.xPos][opponent.yPos] = opponent;
			} else {
				opponents.remove(opponent);
			}
		}
		//Adding all the artefacts to grid
		for (int index = 0; index < artefacts.size(); index++) {
			Artefact artefact = artefacts.get(index);
			grid[artefact.xPos][artefact.yPos] = artefact;
		}
	}
	//Displaying the maze
	public void displayMaze() {
		refreshGrid();

		System.out.print("  ");
		for (int i = 0; i < SIZE; i++) {
			System.out.print(i + " ");

		}
		System.out.print("\n  ");
		for (int i = 0; i < SIZE; i++) {
			System.out.print("- ");
		}

		System.out.println("");

		for (int i = 0; i < SIZE; i++) {
			System.out.print(i + "|");

			for (int j = 0; j < SIZE; j++) {
				if (grid[i][j] == null)
					System.out.print(" |");
				else
					System.out.print(grid[i][j].appearance + "|");
			}
			System.out.print("\n  ");
			for (int k = 0; k < SIZE; k++) {
				System.out.print("- ");
			}
			System.out.println("");
		}

	}

	//Returns false if outside the bounds of the maze
	public boolean withinGrid(int xPos, int yPos) {
		return (xPos >= 0 && xPos < SIZE && yPos >= 0 && yPos < SIZE) ? true
				: false;
	}

}